<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Teacher_Rating extends Controller_Environment_Teacher
{
    public function before() {
        parent::before();

        // make it possible to download files
        Cookie::set('fD', 'true');
    }

    // Получить из кука SGID выбранную ранее группу для данной дисциплины
    private function getFilterGroupID($disciplineID) {
        $groups = json_decode(Cookie::get('SGID', null), true);
        if ($groups && isset($groups[$disciplineID])) {
            return $groups[$disciplineID];
        }
        return 0;
    }

    // Шапка таблицы: структура УКД (модули и мероприятия)
    private function getStructure($id, $type) {
        $structure = Model_Map::getRoadmap($id, $type);

        if ($structure->count() == 0) {
            throw HTTP_Exception::factory(404, "Страница не найдена");
        }

        $structureHandled = [];
        $maxRate = $i = $try = 0; // try = 1 - экзамен, = 2, 3 - пересдачи
        $lastModuleID = -1;

        foreach ( $structure as $row ) {

            if ( $row['ModuleID'] != $lastModuleID ) {
                $lastModuleID = $row['ModuleID'];
                $structureHandled[++$i] = [
                    'SubmodulesCount' => 0,
                    'MaxRate' => 0,
                    'ModuleTitle' => $row['ModuleName'],
                    'ModuleType' =>  $row['ModuleType'],
                ];
            }

            $j = $structureHandled[$i]['SubmodulesCount'] += 1;
            $structureHandled[$i]['MaxRate'] += (int) $row['MaxRate'];

            $structureHandled[$i][$j] = $row;
            $curSubmodule = &$structureHandled[$i][$j];
            $curSubmodule['MaxRate'] = (int) $curSubmodule['MaxRate'];

            if ($type == "rate") {
                $curSubmodule['Title'] = $row['SubmoduleName'];
                $maxRate += $row['MaxRate'];
            } else { //$type == "exam"
                if ($row['ModuleType'] == 'extra') {
                    $curSubmodule['Title'] = 'Добор баллов';
                    $maxRate += $row['MaxRate'];
                } else {
                    if ($try === 0) {
                        $curSubmodule['Title'] = 'Основная сдача';
                    } else {
                        $curSubmodule['Title'] = 'Пересдача ' . $try;
                    }
                    $try++;
                }
            }
        }
        $structureHandled['ModulesCount'] = $i;
        $structureHandled['MaxRate'] = (int) $maxRate;
        return $structureHandled;
    }

    private function processGroupInfo($groupInfo) {
        //GroupID, GroupNum, GradeNum, isAttached etc.
        $out = $groupInfo;
        $degree = $groupInfo['Degree'];

        //Формирование заголовка курса
        $gradeNum = $groupInfo['GradeNum'];
        if ( $degree == 'bachelor' || $degree == 'specialist' )
        {
            $out['GradeTitle'] = $gradeNum . ' курс';
        } else if ( $degree == 'master' ) {
            $out['GradeTitle'] = 'Магистратура, ' . $gradeNum . ' год';
        }

        //Формирование заголовка группы
        $out['GroupTitle'] = $out['GradeTitle'] . " " . $groupInfo['GroupNum'] . " группа";
        return $out;
    }

    private function getRatesForRatingPage($info, &$rateResult) {
        $rateResult = 0;
        $rates = [];

        foreach ($info as $row) {
            $rateResult += $row['Rate'];
            $moduleType = $row['ModuleType'];

            switch ( $moduleType ) {
                case 'exam':
                    $submoduleID = -1;
                break;
                case 'extra':
                    $submoduleID = -2;
                break;
                default:
                    $submoduleID = $row['SubmoduleID'];
            }
            $rates[] = [
                'Type'        => $moduleType,
                'SubmoduleID' => $submoduleID,
                'Rate'        => $row['Rate'],
            ];

        }
        return $rates;
    }

    private function correctExtra(&$curStudent, $examType, $lastExtraIndex, $firstEmptyExtraIndex, $totalExtraRate) {
        $bottomLimit = 0;
        $maxExtraRate = 0;
        $topLimit = ($examType == 'exam') ? 38 : 60;

        if ($curStudent['RateSemesterResult'] >= $bottomLimit &&
            $curStudent['RateSemesterResult'] < $topLimit
        ) // студент задолженик
        {
            $maxExtraRate = $topLimit - $curStudent['RateSemesterResult'];
        }
        if ($lastExtraIndex >= 0) {
            $curStudent['Rates'][$lastExtraIndex]['MaxRate'] = $maxExtraRate - $totalExtraRate
                + $curStudent['Rates'][$lastExtraIndex]['Rate'];
        }
        if ($firstEmptyExtraIndex >= 0) {
            $curStudent['Rates'][$firstEmptyExtraIndex]['MaxRate'] = $maxExtraRate - $totalExtraRate;
        }
    }

    private function getRatesForExamPage(&$curStudent, $rate, $examType, $disciplineID) {
        $rowIndex = 0;
        $lastExam = $lastExtraIndex = $lastNilExam = $firstEmptyExtra = -1;
        $rateExtra = 0;

        $curStudent['RateSemesterResult'] = 0;
        foreach ($rate as $curRate) {

            if (($curRate['ModuleType'] == 'exam') or ($curRate['ModuleType'] == 'extra')) {
                $curStudent['Rates'][$rowIndex] = [
                    'SubmoduleID'      => $curRate['SubmoduleID'],
                    'Rate'             => $curRate['Rate'],
                    'ModuleType'       => $curRate['ModuleType'],
                    'ExamPeriodOption' => $curRate['ExamPeriodOption'],
                ];
            }

            $rate = $curRate['Rate'];

            switch ($curRate['ModuleType']) {
                case 'regular':
                    $curStudent['RateSemesterResult'] += $rate;
                    break;
                case 'exam':
                    if (!is_null($rate)) {
                        if ($lastExam >= 0) {
                            $curStudent['Rates'][$lastExam]['Block'] = 'True';
                        }
                        $lastExam = $rowIndex;
                    } else {
                        if ($lastNilExam < 0) {
                            $lastNilExam = $rowIndex;
                        } else {
                            $curStudent['Rates'][$rowIndex]['Block'] = 'True';
                        }
                    }
                    break;
                case 'bonus':
                    $curStudent['Bonus'] = $rate;
                    break;
                case 'extra':
                    if ($rate) {
                        if ($lastExtraIndex >= 0) {
                            $curStudent['Rates'][$lastExtraIndex]['Block'] = 'True';
                        }
                        $lastExtraIndex = $rowIndex;
                        $curStudent['Rate'] += $curRate['Rate'];
                        $rateExtra += $curRate['Rate'];
                    } else {
                        if ($firstEmptyExtra < 0) {
                            $firstEmptyExtra = $rowIndex;
                        } else {
                            $curStudent['Rates'][$rowIndex]['Block'] = 'True';
                        }
                    }
                    break;
                default:
                    throw HTTP_Exception::factory(500, "Некорректный тип модуля!");
            }
            $rowIndex++;
        }

//        $total = Model_Rating->GetStudentRate($curStudent['ID'], $disciplineID);
//        $total = ($total[0])?$total[0]['Num']:0;
//        if ($total)
//            $total = 0;

        $curStudent['RateResult'] = $curStudent['RateSemesterResult'] + $rateExtra;
        if ($lastExam >= 0) {
            $curStudent['RateResult'] += $curStudent['Rates'][$lastExam]['Rate'];
        }
        $this->correctExtra($curStudent, $examType, $lastExtraIndex, $firstEmptyExtra, $rateExtra);
    }

    /**
     * @param $page_type : rating, exam
     * @throws HTTP_Exception
     */
    protected function action_rate() {
        $id = $this->request->param('id');
        $pageType = $this->request->param('type'); // 'rate', 'exam'

        $discipline = Model_Discipline::load($id);
        $discipline['GroupID_Filter'] = $this->getFilterGroupID($id);
        $discipline['LocalizedExamType'] = RusLang::tr($discipline->Type);

        // Студенты и их баллы
        $students = Model_Rating::GetStudentsForRating($id);

        $rateHandled = [];
        $groupsHandled = [];
        $groupInd = $studentInd = $curGroupID = 0;

        foreach ($students as $row) {

            if ($curGroupID !== $row['GroupID']) {
                $curGroupID = $row['GroupID'];
                $groupInd++;
                $studentInd = 0;
                $rateHandled[$groupInd] = $this->processGroupInfo($row);
                $groupsHandled[$curGroupID] = $rateHandled[$groupInd]['GroupTitle'];
            }

            // Студенты
            $rateHandled[$groupInd]['Students'][$studentInd] = $row;
            $curStudent = &$rateHandled[$groupInd]['Students'][$studentInd++]; //ID, Name, isAttached
            $curStudent['RateResult'] = 0;

            // Баллы студента
            if ($pageType == 'rate') {
                $rates_raw = Model_Rating::getRates($id, $row['ID']);
                $curStudent['Rates'] = $this->getRatesForRatingPage($rates_raw, $curStudent['RateResult']);
            } else {
                $rate = Model_Rating::getExamRates($id, $row['ID']);
                $this->getRatesForExamPage($curStudent, $rate, $discipline->Type, $id);
            }
        }

        $twigType = ($pageType == 'rate') ? 'rating' : $pageType;
        // Шапка таблицы: структура УКД (модули и мероприятия)
        $this->twig->set([
            'groups'          => $groupsHandled,
            'rateTable'       => $rateHandled,
            'headerRate'      => $this->getStructure($id, $pageType),
            'Discipline'      => $discipline,
            'Discipline_JSON' => json_encode($discipline),
        ])->set_filename("teacher/" . $twigType);
    }


}
            